
ShowAttackDistance = ISUIElement:derive("ShowAttackDistance")

function ShowAttackDistance:initialize()
    ISUIElement.initialise(self);
end

function ShowAttackDistance:renderIsoCircle(playerNum, posX, posY, posZ, ray, r, g, b, a)
    if luautils.renderIsoCircle then--use tchernolib for mask and thickness
        luautils.renderIsoCircle(playerNum, posX, posY, posZ, ray, r, g, b, a)
    else
        self:renderIsoCircleLocal(playerNum, posX, posY, posZ, ray, r, g, b, a)
    end
end

function ShowAttackDistance:renderIsoCircleLocal(playerNum, posX, posY, posZ, ray, r, g, b, a)
    local angularStep =  0.1163552834662886D;

    for angularIter = 0, 6.283185307179586D, angularStep do--from 0 to 2 PI
        local xStart = posX + ray * Math.cos(angularIter);
        local yStart = posY + ray * Math.sin(angularIter);
        local xEnd   = posX + ray * Math.cos(angularIter + angularStep);
        local yEnd   = posY + ray * Math.sin(angularIter + angularStep);
        local xScreen1  = isoToScreenX(playerNum, xStart, yStart, posZ);
        local yScreen1  = isoToScreenY(playerNum, xStart, yStart, posZ);
        local xScreen2  = isoToScreenX(playerNum, xEnd  , yEnd  , posZ);
        local yScreen2  = isoToScreenY(playerNum, xEnd  , yEnd  , posZ);
        --LineDrawer.drawLine(xScreen1, yScreen1, xScreen2, yScreen2, r, g, b, a, var8);
        self:drawLine2( xScreen1, yScreen1, xScreen2, yScreen2, a, r, g, b)
    end
end

local bh = nil
function ShowAttackDistance:render()
    if self.playerObj:isAiming() and ShowAttackDistance.isShowAttackDistance(self.playerObj) then
        local weaponItem = self.playerObj:getPrimaryHandItem();
        if not weaponItem or not instanceof(weaponItem,'HandWeapon') then
            if not bh then bh = InventoryItemFactory.CreateItem("Base.BareHands") end
            weaponItem = bh
        end
        if weaponItem then 
            local rayMin  = weaponItem:getMinRange()
            local rayMax  = weaponItem:getMaxRange(self.playerObj)
            local rayRangedDebuff = weaponItem:getMinRangeRanged()
        
            local posX = self.playerObj:getX();
            local posY = self.playerObj:getY();
            local posZ = self.playerObj:getZ();
            
            local r,g,b,alpha = 0.8,0.8,0.8,1
            
            self:renderIsoCircle(self.playerNum, posX, posY, posZ, rayMin, r, g, b, alpha)
            self:renderIsoCircle(self.playerNum, posX, posY, posZ, rayMax, r, g, b, alpha)
            
            if rayRangedDebuff > 0 then
                local r,g,b,alpha = 0.8,0.4,0.2,1
                self:renderIsoCircle(self.playerNum, posX, posY, posZ, rayRangedDebuff, r, g, b, alpha)
            end
        end
    end
end

function ShowAttackDistance:new(playerNum,playerObj)
    local posX = getPlayerScreenLeft(playerNum);
    local posY = getPlayerScreenTop(playerNum);
    local o = {};
    o = ISUIElement:new(posX, posY, 1, 1);
    setmetatable(o, self);
    self.__index = self;
    
    o.playerNum = playerNum
    o.playerObj = playerObj
    
    return o
end

function ShowAttackDistance.onCreatePlayer(playerNum, playerObj)
    if not playerObj then return end
    if playerObj.getIsNPC and playerObj:getIsNPC() then return end --no web on NPCs
    
    ShowAttackDistance.cache = ShowAttackDistance.cache or {}
    
    if ShowAttackDistance.cache[playerNum] then
        ShowAttackDistance.cache[playerNum]:removeFromUIManager();
        ShowAttackDistance.cache[playerNum] = nil;
    end
    
    local newWeb = ShowAttackDistance:new(playerNum,playerObj)
    newWeb:initialize();
    newWeb:instantiate();
    newWeb:addToUIManager();
    ShowAttackDistance.cache[playerNum] = newWeb;
end
function ShowAttackDistance.onCharacterDeath(characterObj)
    if characterObj and characterObj.getPlayerNum then
        local playerNum = characterObj:getPlayerNum()
        if ShowAttackDistance.cache and ShowAttackDistance.cache[playerNum] then
            ShowAttackDistance.cache[playerNum]:removeFromUIManager();
            ShowAttackDistance.cache[playerNum] = nil;
        end
    end
end

Events.OnCreatePlayer.Add(ShowAttackDistance.onCreatePlayer);
Events.OnCharacterDeath.Add(ShowAttackDistance.onCharacterDeath);